#!/usr/bin/perl -w

use strict;
use BerkeleyDB;

use constant DBDIR	=> '/home/jabber/presence/db';
use constant IMGDIR	=> '/home/jabber/presence/img';
my $max_age = 60;

my (%params, $DB);

cgi_params(\%params);
db_init();

#fatal('try again later');

fatal('user not specified') if not $params{user};

my $presence = lc db_get($params{user});
$presence ||= 'no-such-user';
$presence =~ s/^(\d+) +//;		# strip the timestamp
my $last_modified = $1;
$presence = $1 if $presence =~ /^online ([a-z-]+)/;

open(LOG, '>>' . DBDIR . '/log') or fatal("cannot open log: $!");
print LOG time . " $params{user} $presence\n";
close LOG;

if (0 and $presence eq 'no-such-user') {
	print "Status: 404 Not found\n"
		. "Server: presence.pl\n"
		. "Content-Type: text/plain\n"
		. "Cache-Control: max-age=$max_age, public\n\n"
		. "User $params{user} is not subscribed to this service.\n"
		. "Please read http://jabber.linux.it/presence/ and add"
		. " presence\@jabber.linux.it to your roster if you want"
		. " to use this service\n";
	exit 0;
}

# use a longer time for non-existing users
$max_age = 60 * 30 if $presence eq 'no-such-user';

# some users have linked the image in a stupid web board
$max_age = 60 * 60
	if $ENV{HTTP_REFERER} and $ENV{HTTP_REFERER} =~
 m#(?:^http://(?:www\.autocentrum\.pl|www\.miranda\.kom\.pl|miranda\.kom\.pl)|/viewtopic\.php)/#;

my $lmheader = '';
$lmheader = "Last-Modified: " . http_date($last_modified) . "\n"
	if $last_modified;

my ($mime_type, $data);
if (exists $params{text}) {
	$data = "$presence\n";
	$mime_type = 'text/plain';
} else {
	read_image($presence, \$data);
	$mime_type = 'image/png';
}

print "Content-Type: $mime_type\n"
	. "Content-Length: " . length($data) . "\n"
	. $lmheader
	. "Cache-Control: max-age=$max_age, public\n\n"
	. $data;
exit 0;

sub db_init {
	my $env = BerkeleyDB::Env->new(
		-Flags	=> DB_CREATE|DB_INIT_MPOOL|DB_INIT_CDB,
		-Home	=> DBDIR,
		-ErrFile => DBDIR . '/err.log',
		-Verbose => 1|2|4|8,
	);
	fatal("cannot create the DB environment: $BerkeleyDB::Error")
		if not defined $env;

	$DB = BerkeleyDB::Hash->new(
		-Env => $env,
		-Flags => DB_RDONLY,
		-Filename => 'presence.db',
	);
	fatal("cannot open the DB: $BerkeleyDB::Error") if not defined $DB;
}

sub db_get {
	my ($key) = @_;
	my $value;

	my $r = $DB->db_get($key, $value);
	fatal("db_get($key, $value): $r") if $r and $r !~ /^DB_NOTFOUND/;
	return undef if $r;
	return $value;
}

sub cgi_params {
	my ($params) = @_;

	foreach (split(/[&;]/, $ENV{QUERY_STRING} || '')) {
		s/\+/ /g;
		s/%([A-Fa-f0-9]{2})/pack('c', hex($1))/ge;
		my ($key, $val) = split(/=/, $_, 2);
		next unless /^[-_@\.\w\d]+?=[-_@\.\w\d]*?$/;	# is this needed?
		$params->{$key} = $val;
	}
}

sub http_date {
	my ($time) = @_;

	my @MON = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
	my @WDAY = qw/Sun Mon Tue Wed Thu Fri Sat/;

	my ($sec, $min, $hour, $mday, $mon, $year, $wday) = gmtime($time || time);
	return sprintf("%s, %02d %s %04d %02d:%02d:%02d GMT",
		$WDAY[$wday], $mday, $MON[$mon], $year + 1900, $hour, $min, $sec);
}

sub fatal {
	my $t = shift || 'unknown error';

	print "Status: 500\n"
		. "Server: presence.pl\n"
		. "Content-Type: text/plain\n\n"
		. "A fatal error happened:\n$t\n";
	exit 0;
}

sub read_image {
	my ($type, $image) = @_;

	open(IMAGE, IMGDIR . "/$type.png") or fatal("cannot open $type.png");
	local $/ = undef;
	$$image = <IMAGE>;
	close IMAGE;
}
