#!/usr/bin/perl -w

use BerkeleyDB;
use POSIX;
use strict;

use constant DATADIR	=> '/home/jabber/presence/db';

my $DB;

db_init();

my $cursor = $DB->db_cursor;
my $key = '';
my $value = '';
while ($cursor->c_get($key, $value, DB_NEXT) == 0) {
	my ($timestamp, $status) = split(/\s+/, $value);
	my $date = POSIX::strftime('%Y-%m-%d %H:%M', localtime $timestamp);
	printf("%-40s%-18s%s\n", $key, $date, $status);
}

exit 0;

sub db_init {
	my $env = BerkeleyDB::Env->new(
		-Flags	=> DB_INIT_MPOOL|DB_INIT_CDB,
		-Home	=> DATADIR,
		-ErrFile => DATADIR . '/err.log',
		-Verbose => 1|2|4|8,
	);
	die "Cannot create DB environment: $BerkeleyDB::Error" if not defined $env;

	$DB = BerkeleyDB::Hash->new(
		-Env => $env,
		-Flags => DB_RDONLY,
		-Filename => 'presence.db',
	);
	die "Cannot open DB: $BerkeleyDB::Error" if not defined $DB;
}

sub db_del {
	my ($key) = @_;

	my $r = $DB->db_del($key);
	die "db_del($key): $r" if $r and $r !~ /^DB_NOTFOUND/;
}

sub db_get {
	my ($key) = @_;
	my $value;

	my $r = $DB->db_get($key, $value);
	die $r if $r and $r !~ /^DB_NOTFOUND/;
	return undef if $r;
	return $value;
}

